Re: WARNING in memory_failure() at include/linux/huge_mm.h:635 triggered

From: jane . chu

Date: Wed Feb 04 2026 - 21:01:19 EST




On 2/4/2026 1:41 PM, Zi Yan wrote:
On 4 Feb 2026, at 16:37, David Hildenbrand (arm) wrote:

On 2/4/26 22:08, Zi Yan wrote:
On 4 Feb 2026, at 14:18, David Hildenbrand (arm) wrote:

On 2/4/26 18:41, Zi Yan wrote:


More details:
later at sg_vma_fault(), the driver just handles a page fault by supplying
a subpage from a pre-allocated compound page[3]. We then get a large folio
without !CONFIG_TRANSPARENT_HUGEPAGE.

We can identify such non-folio (but compound) things by looking at PG_large_rmappable IIRC.

OK, back to the issue. The patch below should fix the issue?

Hi 是参差,

Can you test it?


<snip>
I think you have to test for folio_test_large() before testing folio_test_large_rmappable().

Oh, forgot that. Thanks.


From 8dda4bba9964890462eca3ef3cce57bb4fab8313 Mon Sep 17 00:00:00 2001
From: Zi Yan <ziy@xxxxxxxxxx>
Date: Wed, 4 Feb 2026 16:04:19 -0500
Subject: [PATCH] mm/memory_failure: reject unsupported non-folio compound page

Signed-off-by: Zi Yan <ziy@xxxxxxxxxx>
---
mm/memory-failure.c | 8 ++++++--
1 file changed, 6 insertions(+), 2 deletions(-)

diff --git a/mm/memory-failure.c b/mm/memory-failure.c
index 825c706ac576..137c67fda57e 100644
--- a/mm/memory-failure.c
+++ b/mm/memory-failure.c
@@ -2440,9 +2440,13 @@ int memory_failure(unsigned long pfn, int flags)

folio = page_folio(p);

/* filter pages that are protected from hwpoison test by users */
folio_lock(folio);
- if (hwpoison_filter(p)) {
+ if (hwpoison_filter(p) ||
+ (folio_test_large(folio) && !folio_test_large_rmappable(folio))) {

Just curious, would this filter out pte-mapped THP/mTHP folios?

ClearPageHWPoison(p);
folio_unlock(folio);
folio_put(folio);

thanks,
-jane
@@ -2440,9 +2440,13 @@ int memory_failure(unsigned long pfn, int flags)
folio = page_folio(p);

- /* filter pages that are protected from hwpoison test by users */
+ /*
+ * filter pages that are protected from hwpoison test by users
+ * or unsupported non folio compound pages
+ */
folio_lock(folio);